@@ -1,7 +1,8 @@
-from .computation_engine import (apply, join, summarize,
-                                 apply_local, join_local, summarize_local,
+from .computation_engine import (apply, join, summarize, fullstream,
+                                 apply_local, join_local, summarize_local, fullstream_local,
+                                 apply_debug, join_debug, summarize_debug, fullstream_debug,
                                  Engine, output, DelayedException,
-                                 fullstream, fullstream_local)
+                                 )
 
 from .fs_cache import CacheBuilder, LocalCacheBuilder
 
