@@ -108,22 +108,37 @@ class Engine(object):
             raise Exception("can not set number of processes to %d" % n)
 
     @staticmethod
-    def run_async(f, args, callback, run_local=False):
+    def run_async(f, args, callback, run_local=False, debug=False):
         """ f is not implemented for handling enumerated items, so we exctract the items
         and construct a 'new enumeration' for the result.
         This is needed in order to ensure that the final result of the computations have
         a fixed order although their computation order might be different due to the
-        concurrent processing."""
+        concurrent processing.
+
+        in debug mode the computation runs local but goes through the pickling/unpickling loop
+        this enables use of debugger and allows detection of pickling problems
+
+        """
         # we use eval_pickled in both cases to make shure we have the same kind of error handling
         # for local and remote execution. this supports debugging.
-        f_s = dill.dumps(f)
-        args_s = dill.dumps(args)
-        if run_local or Engine.pool is None:
+
+        if debug:
+            # run computation local so we can use debugger
+            # but use the pickling / unpickling procedure to detect pickling problems
+            f_s = dill.dumps(f)
+            args_s = dill.dumps(args)
             result = eval_pickled(f_s, args_s)
             callback(result)
         else:
-            Engine.pool.apply_async(eval_pickled, (f_s, args_s),
-                                    callback=callback)
+            run_local = run_local is True or Engine.pool is None
+            if run_local:
+                result = f(*args)
+                callback(result)
+            else:
+                f_s = dill.dumps(f)
+                args_s = dill.dumps(args)
+                Engine.pool.apply_async(eval_pickled, (f_s, args_s),
+                                        callback=callback)
 
 
 class DataStream(object):
@@ -178,9 +193,10 @@ class Source(DataStream):
 
 class Computation(DataStream):
 
-    def __init__(self, f, inputs, run_local=False):
+    def __init__(self, f, inputs, run_local=False, debug=False):
         super(Computation, self).__init__()
         self.run_local = run_local
+        self.debug = debug
         self.f = f
         self.input_streams = self._setup_inputs(inputs)
         self._size = self.compute_size()
@@ -214,7 +230,7 @@ class Computation(DataStream):
                 forward_result(arg.value)
                 return
         args = tuple(a for (__, a) in args)
-        Engine.run_async(self.f, args, forward_result, self.run_local)
+        Engine.run_async(self.f, args, forward_result, self.run_local, self.debug)
 
 
 class OutputNode(DataStream):
@@ -306,11 +322,14 @@ class JoinComputation(Computation):
                 time.sleep(0.001)
 
 
-class SummarizeComputation(Computation):
+class SingleInputComputation(Computation):
 
-    def __init__(self, f, inputs, run_local=False):
+    def __init__(self, f, inputs, run_local=False, debug=False):
         assert len(inputs) == 1, "summarize computations only accept one input stream"
-        super(SummarizeComputation, self).__init__(f, inputs, run_local)
+        super(SingleInputComputation, self).__init__(f, inputs, run_local, debug)
+
+
+class SummarizeComputation(SingleInputComputation):
 
     def compute_size(self):
         return 1
@@ -330,11 +349,7 @@ class SummarizeComputation(Computation):
         self._start_async_computation_for(0, (arg,))
 
 
-class FullStreamComputation(Computation):
-
-    def __init__(self, f, inputs, run_local=False):
-        assert len(inputs) == 1, "full stream computations only accept one input stream"
-        super(FullStreamComputation, self).__init__(f, inputs, run_local)
+class FullStreamComputation(SingleInputComputation):
 
     def compute_size(self):
         return self.input_streams[0].size()
@@ -361,7 +376,7 @@ class FullStreamComputation(Computation):
             for i, item in enumerate(result):
                 self.put(EnumeratedItem(i, item))
 
-        Engine.run_async(self.f, (values,), forward_result, self.run_local)
+        Engine.run_async(self.f, (values,), forward_result, self.run_local, self.debug)
 
 
 class ZipComputation(Computation):
@@ -453,68 +468,31 @@ class ZipComputation(Computation):
                     seen.add(ni)
 
 
-def apply(inner):
-    @functools.wraps(inner)
-    def wrapped(*args):
-        return ZipComputation(inner, args)
-    wrapped.inner = inner
-    return wrapped
+def create_apply_decorator(clz, debug, run_local):
+    def decorator(inner):
+        @functools.wraps(inner)
+        def wrapped(*args):
+            return clz(inner, args, debug=debug, run_local=run_local)
+        wrapped.inner = inner
+        return wrapped
+    return decorator
 
 
-def apply_local(inner):
-    @functools.wraps(inner)
-    def wrapped(*args):
-        return ZipComputation(inner, args, run_local=True)
-    wrapped.inner = inner
-    return wrapped
+apply = create_apply_decorator(ZipComputation, False, False)
+apply_local = create_apply_decorator(ZipComputation, False, True)
+apply_debug = create_apply_decorator(ZipComputation, True, False)
 
+join = create_apply_decorator(JoinComputation, False, False)
+join_local = create_apply_decorator(JoinComputation, False, True)
+join_debug = create_apply_decorator(JoinComputation, True, False)
 
-def join(inner):
-    @functools.wraps(inner)
-    def wrapped(*args):
-        return JoinComputation(inner, args)
-    wrapped.inner = inner
-    return wrapped
+summarize = create_apply_decorator(SummarizeComputation, False, False)
+summarize_local = create_apply_decorator(SummarizeComputation, False, True)
+summarize_debug = create_apply_decorator(SummarizeComputation, True, False)
 
-
-def join_local(inner):
-    @functools.wraps(inner)
-    def wrapped(*args):
-        return JoinComputation(inner, args, run_local=True)
-    wrapped.inner = inner
-    return wrapped
-
-
-def summarize(inner):
-    @functools.wraps(inner)
-    def wrapped(*args):
-        return SummarizeComputation(inner, args)
-    wrapped.inner = inner
-    return wrapped
-
-
-def summarize_local(inner):
-    @functools.wraps(inner)
-    def wrapped(*args):
-        return SummarizeComputation(inner, args, run_local=True)
-    wrapped.inner = inner
-    return wrapped
-
-
-def fullstream(inner):
-    @functools.wraps(inner)
-    def wrapped(*args):
-        return FullStreamComputation(inner, args)
-    wrapped.inner = inner
-    return wrapped
-
-
-def fullstream_local(inner):
-    @functools.wraps(inner)
-    def wrapped(*args):
-        return FullStreamComputation(inner, args, run_local=True)
-    wrapped.inner = inner
-    return wrapped
+fullstream = create_apply_decorator(FullStreamComputation, False, False)
+fullstream_local = create_apply_decorator(FullStreamComputation, False, True)
+fullstream_debug = create_apply_decorator(FullStreamComputation, True, False)
 
 
 def output(inner):
